--[[-------------------------------------------------------------------------
	InterruptHelper
	Copyright (C) 2011  Morsker
	All rights reserved.
	Please contact Morsker through PM on wowace.com.

	Helps with interrupt notifications and announcements.
---------------------------------------------------------------------------]]

local InterruptHelper = InterruptHelper
local db = InterruptHelper.db

-- GLOBALS: LibStub
-- GLOBALS: INTERFACEOPTIONS_ADDONCATEGORIES
-- GLOBALS: InterfaceAddOnsList_Update
-- GLOBALS: tremove
-- GLOBALS: strmatch

local annSayFormatDesc = [[
Format for announcements.
$s = spell name
$S = spell link
]]

local raidChatValues = {
	["NONE"] = "None",
	["SAY"] = "Say",
	["WHISPER"] = "Whisper",
	["CHANNEL"] = "Channel",
	["PARTY"] = "Party",
	["RAID"] = "Raid",
}

local partyChatValues = {
	["NONE"] = "None",
	["SAY"] = "Say",
	["WHISPER"] = "Whisper",
	["CHANNEL"] = "Channel",
	["PARTY"] = "Party",
}

local soloChatValues = {
	["NONE"] = "None",
	["SAY"] = "Say",
	["WHISPER"] = "Whisper",
	["CHANNEL"] = "Channel",
}

local battlegroundChatValues = {
	["NONE"] = "None",
	["SAY"] = "Say",
	["WHISPER"] = "Whisper",
	["CHANNEL"] = "Channel",
	["BATTLEGROUND"] = "Battleground",
}

local function SetGeneric(info, val)
	local setting = info[#info]
	db.profile[setting] = val
end

local function DisabledChannel(info)
	return not db.profile.annAnnounce
end

local function MakeGap(order)
	return {
		order = order,
		type = "description",
		name = "",
		width = "full",
	}
end

local function MakeChatSelect(order, name, chatValues)
	return {
		type = "select",
		name = name,
		order = order,
		values = chatValues,
		disabled = DisabledChannel,
		set = SetGeneric,
	}
end

local function MakeChatArg(order, base)
	return {
		type = "input",
		name = function(info)
			if db.profile[base] == "WHISPER" then
				return "Player"
			elseif db.profile[base] == "CHANNEL" then
				return "Channel"
			end
		end,
		order = order,
		disabled = DisabledChannel,
		hidden = function(info)
			return not (db.profile[base] == "WHISPER" or db.profile[base] == "CHANNEL")
		end,
		get = function(info)
			if db.profile[base] == "WHISPER" then
				return db.profile[base.."Player"]
			elseif db.profile[base] == "CHANNEL" then
				return db.profile[base.."Channel"]
			end
		end,
		set = function(info, val)
			if db.profile[base] == "WHISPER" then
				db.profile[base.."Player"] = val
			elseif db.profile[base] == "CHANNEL" then
				db.profile[base.."Channel"] = val
			end
		end,
	}
end

function InterruptHelper:InitOptions()
	local options = {
		type = "group",
		name = "Interrupt Helper",
		childGroups = "tab",
		get = function(info)
			return db.profile[info[#info]]
		end,
		args = {
			announce = {
				type = "group",
				name = "Announce Settings",
				args = {
					annAnnounce = {
						type = "toggle",
						name = "Announce Interrupts",
						desc = "Announce interrupts in the channels of your choice.",
						order = 10,
						set = SetGeneric,
					},
					annSayFormat = {
						type = "input",
						name = "Format",
						desc = annSayFormatDesc,
						order = 20,
						width = "double",
						disabled = DisabledChannel,
						set = SetGeneric,
					},
					_gap_1 = MakeGap(100),
					annSolo = MakeChatSelect(110, "Solo", soloChatValues),
					annSoloArg = MakeChatArg(120, "annSolo"),
					_gap_2 = MakeGap(200),
					annParty = MakeChatSelect(210, "Party", partyChatValues),
					annPartyArg = MakeChatArg(220, "annParty"),
					_gap_3 = MakeGap(300),
					annRaid = MakeChatSelect(310, "Raid", raidChatValues),
					annRaidArg = MakeChatArg(320, "annRaid"),
					_gap_4 = MakeGap(400),
					annBattleground = MakeChatSelect(410, "Battleground", battlegroundChatValues),
					annBattlegroundArg = MakeChatArg(420, "annBattleground"),
					_gap_5 = MakeGap(500),
					annCombatZone = MakeChatSelect(510, "Combat Zone", raidChatValues),
					annCombatZoneArg = MakeChatArg(520, "annCombatZone"),
				},
			},
			profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(db),
		},
	}

	-- Remove any stub in the interface options, before adding the real entry
	local categories = INTERFACEOPTIONS_ADDONCATEGORIES
	for i = 1,#categories do
		if categories[i].name == "InterruptHelper" then
			tremove(categories, i)
			break
		end
	end
	InterfaceAddOnsList_Update()
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("InterruptHelper", options)
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("InterruptHelper_" .. options.args.announce.name, options.args.announce)
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("InterruptHelper_" .. options.args.profile.name, options.args.profile)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("InterruptHelper", "InterruptHelper")
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("InterruptHelper_" .. options.args.announce.name, options.args.announce.name, "InterruptHelper")
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("InterruptHelper_" .. options.args.profile.name, options.args.profile.name, "InterruptHelper")
	InterruptHelper.options_table = options
end
